@@ -46,6 +46,13 @@ void gui_tempo_hide();
 void gui_tempo_changed(GtkAdjustment *, gpointer);
 void gui_tpl_changed(GtkAdjustment *, gpointer);
 
+/* Track volumes window functions */
+void gui_trackvolumes_refresh();
+void gui_trackvolumes_show();
+void gui_trackvolumes_hide();
+void gui_trackvolumes_changed();
+void gui_trackvolumes_toggled(GtkToggleButton *, gpointer);
+
 /* Section list window functions */
 void gui_sectionlist_show();
 void gui_sectionlist_hide();
