@@ -22,8 +22,11 @@
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <signal.h>
+#include <stdlib.h>
+#include <time.h>
 #include <sys/time.h>
 #include <unistd.h>
+#include <pthread.h>
 #include "song.h"
 #include "editor.h"
 #include "trackerwidget.h"
@@ -33,95 +36,54 @@
 extern struct editor editor;
 extern GtkWidget *gui_main_tracker;
 
-struct itimerval play_timer;
-
-unsigned short tick;
+static unsigned short tick;
 
 /* Plays the song from the beginning */
 void play_song() {
   editor.mode=MODE_PLAY_SONG;
   editor.section=0;
   editor.playseqpos=0;
-  refresh_playseq_and_block();
-  gettimeofday(&editor.playingstarted, NULL);
-  editor.playedsofar.tv_sec=0;
-  editor.playedsofar.tv_usec=0;
-
-  play_set_timer(editor.song->tempo);
-  setitimer(ITIMER_REAL, &play_timer, NULL);
-  signal(SIGALRM, play_handler);
-
-  stop_notes();
+  player_refresh_playseq_and_block();
+  player_start(0);
 }
 
 /* Continues playing the song from the current position */
 void continue_song() {
   if(editor.mode!=MODE_PLAY_SONG) {
     editor.mode=MODE_PLAY_SONG;
-    
-    gettimeofday(&editor.playingstarted, NULL);
-    
-    play_set_timer(editor.song->tempo);
-    setitimer(ITIMER_REAL, &play_timer, NULL);
-    signal(SIGALRM, play_handler);
-    
-    stop_notes();
+    player_start(1);
   }
 }
 
 /* Plays the block from the beginning */
 void play_block() {
   editor.mode=MODE_PLAY_BLOCK;
-
-  gettimeofday(&editor.playingstarted, NULL);
-  editor.playedsofar.tv_sec=0;
-  editor.playedsofar.tv_usec=0;
-
-  play_set_timer(editor.song->tempo);
-  setitimer(ITIMER_REAL, &play_timer, NULL);
-  signal(SIGALRM, play_handler);
-
-  stop_notes();
+  player_start(0);
 }
 
 /* Continues playing the block from the current position */
 void continue_block() {
   if(editor.mode!=MODE_PLAY_BLOCK) {
     editor.mode=MODE_PLAY_BLOCK;
-    
-    gettimeofday(&editor.playingstarted, NULL);
-    
-    play_set_timer(editor.song->tempo);
-    setitimer(ITIMER_REAL, &play_timer, NULL);
-    signal(SIGALRM, play_handler);
-    
-    stop_notes();
+    player_start(1);
   }
 }
 
 /* Stops the player */
 void stop() {
-  struct timeval tod;
-
   if(editor.mode!=MODE_IDLE) {
     editor.mode=MODE_IDLE;
-    
-    gettimeofday(&tod, NULL);
-    editor.playedsofar.tv_sec+=tod.tv_sec-editor.playingstarted.tv_sec;
-    editor.playedsofar.tv_usec+=tod.tv_usec-editor.playingstarted.tv_usec;
-    if(editor.playedsofar.tv_usec<0) {
-      editor.playedsofar.tv_usec+=1000000;
-      editor.playedsofar.tv_sec--;
-    }
-    
-    setitimer(ITIMER_REAL, NULL, NULL);
-    signal(SIGALRM, SIG_DFL);
+    player_stop();
+    player_stop_notes();
+
+    /* Refresh the GUI accordingly
+     * (FIX: This is called both from player and GUI... dangerous!) */
+    gui_info_refresh();
   } 
-  stop_notes();
 }
 
-/* SIGALRT signal handler for triggering the playroutine */
-void play_handler(int signum) {
+/* Handles one tick by playing all notes scheduled to be played */
+void player_next_tick() {
   Tracker *tracker=(Tracker *)gui_main_tracker;
   int i, j;
   struct timeval tod;
@@ -153,10 +115,8 @@ void play_handler(int signum) {
 	break;
       case EFFECT_STOP:
 	/* Only on last tick */
-	if(tick==editor.song->ticksperline-1) {
+	if(tick==editor.song->ticksperline-1)
 	  stop();
-	  gui_info_refresh();
-	}
 	break;
       case EFFECT_VELOCITY:
 	volume=value;
@@ -166,13 +126,17 @@ void play_handler(int signum) {
 	break;
       case EFFECT_TPL:
 	song_set_tpl(editor.song, value);
+	gdk_threads_enter();
 	gui_info_refresh();
 	gui_tempo_refresh();
+	gdk_threads_leave();
 	break;
       case EFFECT_TEMPO:
 	song_set_tempo(editor.song, value);
+	gdk_threads_enter();
 	gui_info_refresh();
 	gui_tempo_refresh();
+	gdk_threads_leave();
 	break;
       }
     }
@@ -183,7 +147,7 @@ void play_handler(int signum) {
       unsigned char note=block->notes[(block->tracks*tracker->patpos+i)*2]-1;
       unsigned short instrument=block->notes[(block->tracks*tracker->patpos+i)*2+1]-1;
       
-      play_note(instrument, note, volume, i);
+      player_play_note(instrument, note, volume, i);
       
       if(instrument<editor.song->numinstruments &&
 	 editor.song->instruments[instrument]!=NULL)
@@ -214,26 +178,30 @@ void play_handler(int signum) {
 
   switch(posteffect) {
   case EFFECT_END_BLOCK:
-    play_playseq_advance();
-    refresh_playseq_and_block();
+    player_next_playseq();
+    player_refresh_playseq_and_block();
+    gdk_threads_enter();
     tracker_set_pattern(tracker, editor.song->blocks[editor.block]);
     tracker_set_patpos(tracker, 0);
     gui_info_refresh();
     gui_blocklist_refresh();
     gui_playseq_refresh();
+    gdk_threads_leave();
     break;
   case EFFECT_PLAYSEQ_POSITION:
     editor.playseqpos=postvalue;
     if(editor.playseqpos>=editor.song->playseqs[editor.playseq]->length){
       editor.playseqpos=0;
-      play_section_advance();
+      player_next_section();
     }
-    refresh_playseq_and_block();
+    player_refresh_playseq_and_block();
+    gdk_threads_enter();
     tracker_set_pattern(tracker, editor.song->blocks[editor.block]);
     tracker_set_patpos(tracker, 0);
     gui_info_refresh();
     gui_blocklist_refresh();
     gui_playseq_refresh();
+    gdk_threads_leave();
     break;
   default:
     /* Advance in block if ticksperline ticks have passed */
@@ -242,49 +210,44 @@ void play_handler(int signum) {
       if(newpos>=tracker->curpattern->length) {
 	newpos=0;
 	if(editor.mode==MODE_PLAY_SONG)
-	  play_playseq_advance();
-	refresh_playseq_and_block();
+	  player_next_playseq();
+	player_refresh_playseq_and_block();
+	gdk_threads_enter();
 	tracker_set_pattern(tracker, editor.song->blocks[editor.block]);
 	gui_info_refresh();
 	gui_blocklist_refresh();
 	gui_playseq_refresh();
+	gdk_threads_leave();
       }
+      gdk_threads_enter();
       tracker_set_patpos(tracker, newpos);
-      
       gettimeofday(&tod, NULL);
       gui_timer_refresh((int)(editor.playedsofar.tv_sec+tod.tv_sec-editor.playingstarted.tv_sec));
+      gdk_threads_leave();
     }
     break;
   }
 }
 
-void play_playseq_advance() {
+/* Advances in playing sequence and jumps to next section if necessary */
+void player_next_playseq() {
   editor.playseqpos++;
   if(editor.playseqpos>=editor.song->playseqs[editor.playseq]->length) {
     editor.playseqpos=0;
-    play_section_advance();
+    player_next_section();
   }
 }
 
-void play_section_advance() {
+/* Advances in section and jumps to the beginning if necessary */
+void player_next_section() {
   editor.section++;
   if(editor.section>=editor.song->numsections)
     editor.section=0;
 }
 
-/* Sets the timer structure according to a given BPM count */
-void play_set_timer(unsigned short tempo) {
-  /* Tick every 1000000/((BPM/60)*24) usecs */
-  play_timer.it_interval.tv_sec=0;
-  play_timer.it_interval.tv_usec=2500000/tempo;
-  play_timer.it_value.tv_sec=0;
-  play_timer.it_value.tv_usec=2500000/tempo;
-  tick=0;
-}
-
 /* Plays a note using given instrument on a given track */
-void play_note(unsigned short instrument, unsigned char note,
-	       unsigned char volume, unsigned char track) {
+void player_play_note(unsigned short instrument, unsigned char note,
+		      unsigned char volume, unsigned char track) {
   /* Stop currently playing note */
   if(editor.trackstatus[track*4+1]!=-1)
     midi_note_off(editor.trackstatus[track*4], editor.trackstatus[track*4+1],
@@ -316,7 +279,7 @@ void play_note(unsigned short instrument, unsigned char note,
 }
 
 /* Stops notes playing at the moment */
-void stop_notes() {
+void player_stop_notes() {
   int i;
   for(i=0; i<editor.song->maxtracks; i++) {
     if(editor.trackstatus[i*4+1]!=-1) {
@@ -330,23 +293,85 @@ void stop_notes() {
 }
 
 /* Stops all notes playing at the moment */
-void stop_all_notes() {
+void player_stop_all_notes() {
   int i, j;
   for(i=0; i<16; i++)
     for(j=0; j<128; j++)
       midi_note_off(i, j, 127);
 }
 
-/* Refreshes editor.playseq from editor.section and
- * editor.block from editor.playseqpos */
-void refresh_playseq_and_block() {
+/* Refreshes editor.playseq from editor.section and editor.block from
+ * editor.playseqpos */
+void player_refresh_playseq_and_block() {
   if(editor.section>=editor.song->numsections)
     editor.section=0;
   editor.playseq=editor.song->sections[editor.section];
 
   if(editor.playseqpos>=editor.song->playseqs[editor.playseq]->length) {
     editor.playseqpos=0;
-    refresh_playseq_and_block();
+    player_refresh_playseq_and_block();
   } else
     editor.block=editor.song->playseqs[editor.playseq]->blocknumbers[editor.playseqpos];
 }
+
+/* Creates the player thread */
+void player_start(int cont) {
+  player_stop();
+  player_stop_notes();
+
+  gettimeofday(&editor.playingstarted, NULL);
+  if(cont==0) {
+    editor.playedsofar.tv_sec=0;
+    editor.playedsofar.tv_usec=0;
+  }
+
+  editor.threadstatus=pthread_create(&editor.thread, NULL, player_thread,
+				     NULL);
+}
+
+/* Kills the player thread */
+void player_stop() {
+  if(editor.threadstatus==0) {
+    pthread_cancel(editor.thread);
+    editor.threadstatus=-1;
+  }
+}
+
+/* Player thread */
+void player_thread() {
+  struct timeval next, now;
+  struct timespec req, rem;
+
+  tick=0;
+  next.tv_sec=editor.playingstarted.tv_sec;
+  next.tv_usec=editor.playingstarted.tv_usec;
+
+  while(1) {
+    /* Calculate time of next tick (tick every 1000000/((BPM/60)*24) usecs) */
+    next.tv_sec+=(25/editor.song->tempo)/10;
+    next.tv_usec+=((2500000/editor.song->tempo)%1000000);
+    while(next.tv_usec>1000000) {
+      next.tv_sec++;
+      next.tv_usec-=1000000;
+    }
+
+    /* Calculate difference between now and the next tick */
+    gettimeofday(&now, NULL);
+    req.tv_sec=next.tv_sec-now.tv_sec;
+    req.tv_nsec=(next.tv_usec-now.tv_usec)*1000;
+    while(req.tv_nsec<0) {
+      req.tv_sec--;
+      req.tv_nsec+=1000000000;
+    }
+
+    /* Sleep until the next tick if necessary */
+    if(req.tv_sec>=0)
+      while(nanosleep(&req, &rem)==-1);
+
+    /* Handle this tick */
+    player_next_tick();
+
+    /* Check whether this thread should be killed */
+    pthread_testcancel();
+  }
+}
