@@ -218,7 +218,7 @@ void song_delete_playseq(struct song *song, int pos) {
     /* Reallocate the playseqs array */
     song->playseqs=(struct playseq **)realloc(song->playseqs, song->numplayseqs*sizeof(struct playseq *));
 
-    refresh_playseq_and_block();
+    player_refresh_playseq_and_block();
   }
 }
 
@@ -263,7 +263,7 @@ void song_delete_section(struct song *song, int pos) {
     /* Reallocate the sections array */
     song->sections=(unsigned short *)realloc(song->sections, song->numsections*sizeof(unsigned short));
 
-    refresh_playseq_and_block();
+    player_refresh_playseq_and_block();
   }
 }
 
@@ -292,6 +292,7 @@ int song_check_maxtracks(struct song *song) {
     if(song->blocks[i]->tracks>max)
       max=song->blocks[i]->tracks;
   
+  /* If the maximum number of tracks as changed... */
   if(oldmax!=max) {
     /* Reallocate volume array */
     song->trackvolumes=(unsigned char *)realloc(song->trackvolumes,
@@ -307,6 +308,23 @@ int song_check_maxtracks(struct song *song) {
     return 0;
 }
 
+/* Make sure the instrument exists; add instruments if necessary */
+void song_check_instrument(struct song *song, int instrument) {
+  if(instrument>=song->numinstruments || song->instruments[instrument]==NULL) {
+    if(instrument>=song->numinstruments) {
+      int i;
+
+      song->instruments=realloc(song->instruments,
+				(instrument+1)*sizeof(struct instrument *));
+      for(i=song->numinstruments; i<(instrument+1); i++)
+	song->instruments[i]=NULL;
+
+      song->numinstruments=instrument+1;
+    }
+    song->instruments[instrument]=instrument_alloc();
+  }
+}
+
 /* Converts an MMD2 module to a song */
 struct song *song_convert_MMD2(struct MMD2 *mmd) {
   struct song *song;
@@ -468,21 +486,9 @@ struct song *song_convert_MMD2(struct MMD2 *mmd) {
     else
       song->mastervolume=127;
   } else {
-    song->tempo=DEFAULT_TEMPO;
-    song->ticksperline=6;
-    song->numsections=1;
-    song->sections=(unsigned short *)calloc(1, sizeof(unsigned short));
-    song->numplayseqs=1;
-    song->playseqs=(struct playseq **)calloc(1, sizeof(struct playseq *));
-    song->playseqs[0]=playseq_alloc();
-    song->numblocks=1;
-    song->blocks=(struct block **)calloc(1, sizeof(struct block *));
-    song->blocks[0]=block_alloc(4, 64, 1);
-    song->numinstruments=1;
-    song->instruments=(struct instrument **)calloc(1, sizeof(struct instrument *));
-    song->instruments[0]=instrument_alloc();
-    song->maxtracks=4;
-    song->trackvolumes=(char *)calloc(4, sizeof(char));
+    /* Nothing can be done in this case... should never happen though */
+    free(song);
+    song=song_alloc();
   }
 
   return song;
@@ -502,6 +508,7 @@ struct song *song_load(char *filename) {
     mmd=MMD2_load(filename);
     song=song_convert_MMD2(mmd);
     MMD2_free(mmd);
+    /* If it didn't succeed either just allocate a new song */
     if(song!=NULL)
       return song;
     else
